@@ -317,31 +317,71 @@ helps['policy definition create'] = """
                 - name: --rules
                   type: string
                   short-summary: Policy rules in JSON format, or a path to a file containing JSON rules.
+                - name: --management-group
+                  type: string
+                  short-summary: Name of the management group the new policy definition can be assigned in.
+                - name: --subscription
+                  type: string
+                  short-summary: Name or id of the subscription the new policy definition can be assigned in.
             examples:
                 - name: Create a read-only policy.
                   text: |
-                    az policy definition create -n readOnlyStorage --rules '{
-                            "if":
-                            {
-                                "field": "type",
-                                "equals": "Microsoft.Storage/storageAccounts/write"
-                            },
-                            "then":
-                            {
-                                "effect": "deny"
-                            }
-                        }'
-                - name: Create a policy parameter definition with the following example
+                    az policy definition create --name readOnlyStorage --rules '{ \\
+                        "if": \\
+                        { \\
+                            "field": "type", \\
+                            "equals": "Microsoft.Storage/storageAccounts/write" \\
+                        }, \\
+                        "then": \\
+                        { \\
+                            "effect": "deny" \\
+                        } \\
+                    }'
+                - name: Create a policy parameter definition.
                   text: |
-                    az policy definition create -n allowedLocations --rules '{
-                        "allowedLocations": {
-                            "type": "array",
-                            "metadata": {
-                                "description": "The list of locations that can be specified when deploying resources",
-                                "strongType": "location",
-                                "displayName": "Allowed locations"
-                            }
-                        }
+                    az policy definition create --name allowedLocations --rules '{ \\
+                        "if": { \\
+                            "allOf": [ \\
+                                { \\
+                                    "field": "location", \\
+                                    "notIn": "[parameters('listOfAllowedLocations')]" \\
+                                }, \\
+                                { \\
+                                    "field": "location", \\
+                                    "notEquals": "global" \\
+                                }, \\
+                                { \\
+                                    "field": "type", \\
+                                    "notEquals": "Microsoft.AzureActiveDirectory/b2cDirectories" \\
+                                } \\
+                            ] \\
+                        }, \\
+                        "then": { \\
+                            "effect": "deny" \\
+                        } \\
+                    }' \\
+                    --params '{ \\
+                        "allowedLocations": { \\
+                            "type": "array", \\
+                            "metadata": { \\
+                                "description": "The list of locations that can be specified when deploying resources", \\
+                                "strongType": "location", \\
+                                "displayName": "Allowed locations" \\
+                            } \\
+                        } \\
+                    }'
+                - name: Create a read-only policy that can be applied within a management group.
+                  text: |
+                    az policy definition create -n readOnlyStorage --management-group 'MyManagementGroup' --rules '{ \\
+                        "if": \\
+                        { \\
+                            "field": "type", \\
+                            "equals": "Microsoft.Storage/storageAccounts/write" \\
+                        }, \\
+                        "then": \\
+                        { \\
+                            "effect": "deny" \\
+                        } \\
                     }'
 """
 helps['policy definition delete'] = """
@@ -350,7 +390,7 @@ helps['policy definition delete'] = """
 """
 helps['policy definition show'] = """
     type: command
-    short-summary: get a policy definition.
+    short-summary: Show a policy definition.
 """
 helps['policy definition update'] = """
     type: command
@@ -371,13 +411,26 @@ helps['policy set-definition create'] = """
                 - name: --definitions
                   type: string
                   short-summary: Policy definitions in JSON format, or a path to a file containing JSON rules.
+                - name: --management-group
+                  type: string
+                  short-summary: Name of management group the new policy set definition can be assigned in.
+                - name: --subscription
+                  type: string
+                  short-summary: Name or id of the subscription the new policy set definition can be assigned in.
             examples:
                 - name: Create a policy set definition.
                   text: |
-                    az policy set-definition create -n readOnlyStorage --definitions '[
-                            {
-                                "policyDefinitionId": "/subscriptions/mySubId/providers/Microsoft.Authorization/policyDefinitions/storagePolicy"
-                            }
+                    az policy set-definition create -n readOnlyStorage --definitions '[ \\
+                            { \\
+                                "policyDefinitionId": "/subscriptions/mySubId/providers/Microsoft.Authorization/policyDefinitions/storagePolicy" \\
+                            } \\
+                        ]'
+                - name: Create a policy set definition to be used by a subscription.
+                  text: |
+                    az policy set-definition create -n readOnlyStorage --subscription '0b1f6471-1bf0-4dda-aec3-111122223333' --definitions '[ \\
+                            { \\
+                                "policyDefinitionId": "/subscriptions/mySubId/providers/Microsoft.Authorization/policyDefinitions/storagePolicy" \\
+                            } \\
                         ]'
 """
 helps['policy set-definition delete'] = """
@@ -386,7 +439,7 @@ helps['policy set-definition delete'] = """
 """
 helps['policy set-definition show'] = """
     type: command
-    short-summary: get a policy set definition.
+    short-summary: Show a policy set definition.
 """
 helps['policy set-definition update'] = """
     type: command
@@ -403,18 +456,44 @@ helps['policy assignment'] = """
 helps['policy assignment create'] = """
     type: command
     short-summary: Create a resource policy assignment.
+    parameters:
+        - name: --scope
+          type: string
+          short-summary: Scope to which this policy assignment applies.
     examples:
-        - name: Provide rule parameter values with the following example
+        - name: Create a resource policy assignment at scope
           text: |
-                az policy assignment create --policy {PolicyNamed} -p '{
-                    "allowedLocations": {
-                        "value": [
-                            "australiaeast",
-                            "eastus",
-                            "japaneast"
-                        ]
-                    }
+           Valid scopes are management group, subscription, resource group, and resource, for example
+              management group:  /providers/Microsoft.Management/managementGroups/MyManagementGroup
+              subscription:      /subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333
+              resource group:    /subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup
+              resource:          /subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup/providers/Microsoft.Compute/virtualMachines/myVM
+                az policy assignment create --scope '/providers/Microsoft.Management/managementGroups/MyManagementGroup' --policy {PolicyName} -p '{ \\
+                    "allowedLocations": { \\
+                        "value": [ \\
+                            "australiaeast", \\
+                            "eastus", \\
+                            "japaneast" \\
+                        ] \\
+                    } \\
                 }'
+        - name: Create a resource policy assignment and provide rule parameter values.
+          text: |
+                az policy assignment create --policy {PolicyName} -p '{ \\
+                    "allowedLocations": { \\
+                        "value": [ \\
+                            "australiaeast", \\
+                            "eastus", \\
+                            "japaneast" \\
+                        ] \\
+                    } \\
+                }'
+        - name: Create a resource policy assignment with a system assigned identity.
+          text: >
+            az policy assignment create --name myPolicy --policy {PolicyName} --assign-identity
+        - name: Create a resource policy assignment with a system assigned identity. The identity will have 'Contributor' role access to the subscription.
+          text: >
+            az policy assignment create --name myPolicy --policy {PolicyName} --assign-identity --identity-scope /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx --role Contributor
 """
 helps['policy assignment delete'] = """
     type: command
@@ -428,6 +507,29 @@ helps['policy assignment list'] = """
     type: command
     short-summary: List resource policy assignments.
 """
+helps['policy assignment identity'] = """
+    type: group
+    short-summary: Manage a policy assignment's managed identity.
+"""
+helps['policy assignment identity assign'] = """
+    type: command
+    short-summary: Add a system assigned identity to a policy assignment.
+    examples:
+        - name: Add a system assigned managed identity to a policy assignment.
+          text: >
+            az policy assignment identity assign -g MyResourceGroup -n MyPolicyAssignment
+        - name: Add a system assigned managed identity to a policy assignment and grant it the 'Contributor' role for the current resource group.
+          text: >
+            az policy assignment identity assign -g MyResourceGroup -n MyPolicyAssignment --role Contributor --identity-scope /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/MyResourceGroup
+"""
+helps['policy assignment identity show'] = """
+    type: command
+    short-summary: Show a policy assignment's managed identity.
+"""
+helps['policy assignment identity remove'] = """
+    type: command
+    short-summary: Remove a managed identity from a policy assignment.
+"""
 helps['resource'] = """
     type: group
     short-summary: Manage Azure resources.
@@ -507,20 +609,20 @@ helps['resource create'] = """
     examples:
        - name: Create an API app by providing a full JSON configuration.
          text: |
-            az resource create -g myRG -n myApiApp --resource-type Microsoft.web/sites --is-full-object --properties '{
-                        "kind": "api",
-                        "location": "West US",
-                        "properties": {
-                            "serverFarmId": "/subscriptions/{SubID}/resourcegroups/{ResourceGroup}/providers/Microsoft.Web/serverfarms/{ServicePlan}"
-                        }
+            az resource create -g myRG -n myApiApp --resource-type Microsoft.web/sites --is-full-object --properties '{ \\
+                        "kind": "api", \\
+                        "location": "West US", \\
+                        "properties": { \\
+                            "serverFarmId": "/subscriptions/{SubID}/resourcegroups/{ResourceGroup}/providers/Microsoft.Web/serverfarms/{ServicePlan}" \\
+                        } \\
                     }'
        - name: Create a resource by loading JSON configuration from a file.
          text: >
             az resource create -g myRG -n myApiApp --resource-type Microsoft.web/sites --is-full-object --properties @jsonConfigFile
        - name: Create a web app with the minimum required configuration information.
          text: |
-            az resource create -g myRG -n myWeb --resource-type Microsoft.web/sites --properties '{
-                    "serverFarmId":"/subscriptions/{SubID}/resourcegroups/{ResourceGroup}/providers/Microsoft.Web/serverfarms/{ServicePlan}"
+            az resource create -g myRG -n myWeb --resource-type Microsoft.web/sites --properties '{ \\
+                    "serverFarmId":"/subscriptions/{SubID}/resourcegroups/{ResourceGroup}/providers/Microsoft.Web/serverfarms/{ServicePlan}" \\
                 }'
 """
 
@@ -529,6 +631,11 @@ helps['resource update'] = """
     short-summary: Update a resource.
 """
 
+helps['resource wait'] = """
+    type: command
+    short-summary: Place the CLI in a waiting state until a condition of a resources is met.
+"""
+
 helps['resource invoke-action'] = """
     type: command
     short-summary: Invoke an action on the resource.
@@ -545,10 +652,10 @@ helps['resource invoke-action'] = """
          text: >
             az resource invoke-action --action capture \\
               --ids /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Compute/virtualMachines/{VMName} \\
-              --request-body '{
-                "vhdPrefix": "myPrefix",
-                "destinationContainerName": "myContainer",
-                "overwriteVhds": true
+              --request-body '{ \\
+                "vhdPrefix": "myPrefix", \\
+                "destinationContainerName": "myContainer", \\
+                "overwriteVhds": true \\
             }'
 """
 
@@ -635,15 +742,15 @@ helps['group deployment create'] = """
             az group deployment create -g MyResourceGroup --template-uri https://myresource/azuredeploy.json --parameters @myparameters.json
         - name: Create a deployment from a local template file, using parameters from a JSON string.
           text: |
-            az group deployment create -g MyResourceGroup --template-file azuredeploy.json --parameters '{
-                    "location": {
-                        "value": "westus"
-                    }
+            az group deployment create -g MyResourceGroup --template-file azuredeploy.json --parameters '{ \\
+                    "location": { \\
+                        "value": "westus" \\
+                    } \\
                 }'
-        - name: Create a deployment from a local template, using a parameter file and selectively overriding key/value pairs.
+        - name: Create a deployment from a local template, using a local parameter file, a remote parameter file, and selectively overriding key/value pairs.
           text: >
             az group deployment create -g MyResourceGroup --template-file azuredeploy.json \\
-                --parameters @params.json --parameters MyValue=This MyArray=@array.json
+                --parameters @params.json --parameters https://mysite/params.json --parameters MyValue=This MyArray=@array.json
 """
 helps['group deployment export'] = """
     type: command
@@ -686,15 +793,15 @@ helps['deployment create'] = """
             az deployment create --location WestUS --template-uri https://myresource/azuredeploy.json --parameters @myparameters.json
         - name: Create a deployment from a local template file, using parameters from a JSON string.
           text: |
-            az deployment create --location WestUS --template-file azuredeploy.json --parameters '{
-                    "policyName": {
-                        "value": "policy2"
-                    }
+            az deployment create --location WestUS --template-file azuredeploy.json --parameters '{ \\
+                    "policyName": { \\
+                        "value": "policy2" \\
+                    } \\
                 }'
-        - name: Create a deployment from a local template, using a parameter file and selectively overriding key/value pairs.
+        - name: Create a deployment from a local template, using a parameter file, a remote parameter file, and selectively overriding key/value pairs.
           text: >
-            az deployment create --location WestUS --template-file azuredeploy.json \\
-                --parameters @params.json --parameters MyValue=This MyArray=@array.json
+            az deployment create --location WestUS --template-file azuredeploy.json  \\
+                --parameters @params.json --parameters https://mysite/params.json --parameters MyValue=This MyArray=@array.json
 """
 helps['deployment export'] = """
     type: command
